"""Base class to be extended by each generator"""

import os
import shutil
from abc import ABC
from jinja2 import Environment, FileSystemLoader, Undefined
import inquirer
import git

class BaseGenerator(ABC):
    '''Abstract class to be extended by each generator'''

    def replace_in_file(self, file_path, params):
        '''Replace the placeholders in a file with the given params'''

        env = Environment(loader=FileSystemLoader(os.path.dirname(file_path)))
        # pylint: disable=too-few-public-methods
        class SilentUndefined(Undefined):
            '''Custom handler for undefined variables in jinja2'''
            def _fail_with_undefined_error(self, *args, **kwargs):
                return ''
        env.undefined = SilentUndefined
        template = env.get_template(os.path.basename(file_path))
        rendered_template = template.render(params=params)
        with open(file_path, 'w', encoding='utf-8') as file:
            file.write(rendered_template)

    def replace_in_folder(self, folder_path, params):
        '''Runs replace_in_file for each file in the given folder'''
        # pylint: disable=unused-variable
        for root, dirs, files in os.walk(folder_path):
            for file_name in files:
                file_path = os.path.join(root, file_name)
                self.replace_in_file(file_path, params)

    def get_templates(self):
        '''Returns a dictionary with the available templates for the generator'''
        raise NotImplementedError

    def ask_template(self):
        '''Ask the user to select a template from the list returned by get_templates'''
        templates = self.get_templates()

        templates_names = list(templates.keys())
        templates_names.append("Other")

        questions = [
            inquirer.List(
                "template", 
                message="Select one of the available chatbot's templates:",
                choices=templates_names
            ),
        ]

        answers = inquirer.prompt(questions)

        if answers["template"] == "Other":
            template_url_question = [
                inquirer.Text(
                    "template_url", 
                    message="Enter the URL of the template's repository:"
                ),
            ]
            template_url_answers = inquirer.prompt(template_url_question)
            template_url = template_url_answers["template_url"]
            template_name = template_url.split("/")[-1].split(".")[0]
        else:
            template_url = templates[answers["template"]]
            template_name = answers["template"]

        return {
            "template_name": template_name,
            "template_url": template_url
        }

    def clone_template(self, template_url, template_name):
        '''Download a template from github and extract the skeleton'''

        # Clone the template
        git.Repo.clone_from(template_url, os.getcwd() + '/' + template_name)

    def extract_skeleton(self, template_name):
        '''Extract the template skeleton'''
        shutil.move(
            os.getcwd() + '/' + template_name + '/skeleton', os.getcwd() + '/skeleton'
        )

    def ask_template_inputs(self, template_path):
        '''Asks the user the inputs.json questions in the template's folder'''
        if os.path.isfile(template_path + '/inputs.json'):
            with open(template_path + '/inputs.json', 'r', encoding='utf-8') as schema:
                questions = inquirer.load_from_json(schema.read())
                answers = inquirer.prompt(questions)
                return answers
        return {}

    def cleanup(self, template_name):
        '''Removes the base folder from the template'''
        shutil.rmtree(os.getcwd() + '/' + template_name)



"""Dispatches the action to the corresponding chatbot generator"""

import inquirer
from .cli import CliChatbotGenerator
from .telegram import TelegramChatbotGenerator

def create_chatbot():
    """Ask the user for the chatbot's name and interface and create it"""
    questions = [
        inquirer.Text("name", message="Name of your chatbot:"),
        inquirer.List(
            "interface", 
            message="Choose your chatbot's interface:",
            choices=["CLI", "Telegram"]
        ),
    ]
    answers = inquirer.prompt(questions)
    if answers["interface"] == "CLI":
        generator = CliChatbotGenerator()
        generator.create_cli_chatbot(answers["name"])
    elif answers["interface"] == "Telegram":
        generator = TelegramChatbotGenerator()
        generator.create_telegram_chatbot(answers["name"])



"""Telegram chatbot manager"""

import os
import inquirer
from ..base import BaseGenerator

class TelegramChatbotGenerator(BaseGenerator):
    """Telegram chatbot manager"""

    def get_templates(self):
        return {
        'telegram-langchain-chatbot': 'https://github.com/JoseHervas/telegram-langchain-chatbot.git'
    }

    def create_telegram_chatbot(self, name):
        """Generates a new Telegram chatbot"""
        app_path = os.getcwd() + '/' + name
        selected_template = self.ask_template()
        questions = [
                inquirer.List(
                    "token", 
                    message="Do you have a Telegram bot token?",
                    choices=["yes", "no"]
                ),
            ]
        answers = inquirer.prompt(questions)
        if answers["token"] == "yes":
            questions = [
                inquirer.Text("token", message="Enter your Telegram bot token:"),
            ]
            telegram_token = inquirer.prompt(questions)
        self.clone_template(selected_template["template_url"], selected_template["template_name"])
        os.rename(os.getcwd() + '/skeleton', app_path)
        self.cleanup(selected_template["template_name"])
        self.replace_in_folder(app_path, {'chatbot_name': name})
        if telegram_token:
            self.replace_in_folder(app_path, {'telegram_bot_token': telegram_token["token"]})
        print("Yeepay 🎉, your chatbot is ready!")
        print("You can find it in the " + name + " folder")



"""CLI chatbot manager"""

import os
from ..base import BaseGenerator

class CliChatbotGenerator(BaseGenerator):
    """CLI chatbot manager"""

    def get_templates(self):
        return {
        'basic-langchain-template': 'https://github.com/JoseHervas/basic-langchain-chatbot.git',
        'langchain-chatbot-tools': 'https://github.com/JoseHervas/langchain-chatbot-tools'
    }

    def create_cli_chatbot(self, name):
        """Generates a new CLI chatbot"""
        app_path = os.getcwd() + '/' + name
        selected_template = self.ask_template()
        self.clone_template(selected_template["template_url"], selected_template["template_name"])
        template_params = {'chatbot_name': name}
        template_custom_inputs = self.ask_template_inputs(selected_template["template_name"])
        if bool(template_custom_inputs):
            template_params.update(template_custom_inputs)
        self.replace_in_folder(
            selected_template["template_name"] + '/skeleton',
            template_params
        )
        self.extract_skeleton(selected_template["template_name"])
        os.rename(os.getcwd() + '/skeleton', app_path)
        self.cleanup(selected_template["template_name"])
        print("Yeepay 🎉, your chatbot is ready!")
        print("You can find it in the " + name + " folder")



'''Pypi setup file for corby'''

from setuptools import setup, find_packages

setup(
    name="corby",
    version="0.0.13",
    description="⚡ Create your LLMs applications from zero to deploy in minutes ⚡",
    # pylint: disable=consider-using-with
    long_description=open("README.md", encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    author="Jose Hervás Díaz",
    author_email='jhervasdiaz@gmail.com',
    license='MIT',
    packages=find_packages(),
    install_requires=[
        "inquirer",
        "jinja2",
        "gitpython",
    ],
)



#!/usr/bin/env python3

"""Main CLI entrypoint for Corby"""

import argparse
from string import Template
from .generator.chatbot.index import create_chatbot

__version__ = "0.0.13"

# We'll include more features in the future
available_actions = ['new']

available_entities = ['chatbot', 'notebook']

parser = argparse.ArgumentParser(
    usage=Template('corby $available_actions {$available_entities} [options]').substitute(
        available_actions=' | '.join(available_actions),
        available_entities=' | '.join(available_entities)
    )
)

parser.add_argument('action', choices=available_actions, help='Action to perform')
parser.add_argument('entity', choices=available_entities, help='Entity to work with')
parser.add_argument('-v','--version', action='version', version=__version__, help='Show version')
parser.add_argument('-d', '--debug', action='store_true', help='Debug mode')
args = parser.parse_args()

def main():
    """Dispatches the action to the corresponding manager"""
    if args.action:
        if args.action == 'new':
            if args.entity == 'notebook':
                print("Sorry, the notebook generator is not yet available 😥")
            elif args.entity == 'chatbot':
                create_chatbot()

if __name__ == "__main__":
    main()
